@@ -4,7 +4,7 @@ from collections import namedtuple
 from logging import getLogger
 from pathlib import Path
 from tempfile import TemporaryDirectory
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Optional, Tuple, Union, cast
 from urllib.parse import urljoin, urlparse, urlunparse
 
 from .auth import Authentication
@@ -15,6 +15,7 @@ from .utils import check_endpoint, create_md5, get_media_type
 
 logger = getLogger(__name__)
 UploadResult = namedtuple("UploadResult", ("image_id", "movie_id"))
+DEFAULT_ANYMOTION_API_URL = "https://api.customer.jp/anymotion/v1/"
 
 
 class Client(object):
@@ -32,14 +33,12 @@ class Client(object):
 
     def __init__(
         self,
-        client_id: str = os.getenv("ANYMOTION_CLIENT_ID", ""),
-        client_secret: str = os.getenv("ANYMOTION_CLIENT_SECRET", ""),
-        api_url: str = os.getenv(
-            "ANYMOTION_API_URL", "https://api.customer.jp/anymotion/v1/"
-        ),
+        client_id: Optional[str] = None,
+        client_secret: Optional[str] = None,
+        api_url: Optional[str] = None,
         interval: Union[int, float] = 5,
         timeout: Union[int, float] = 600,
-        session: HttpSession = HttpSession(),
+        session: Optional[HttpSession] = None,
     ):
         """Initialize the client.
 
@@ -62,21 +61,28 @@ class Client(object):
         """
         logger.debug("Initializing client.")
 
-        if not isinstance(session, HttpSession):
+        if client_id is None:
+            client_id = os.getenv("ANYMOTION_CLIENT_ID", "")
+        client_id = cast(str, client_id)
+
+        if client_secret is None:
+            client_secret = os.getenv("ANYMOTION_CLIENT_SECRET", "")
+        client_secret = cast(str, client_secret)
+
+        if api_url is None:
+            api_url = os.getenv("ANYMOTION_API_URL", DEFAULT_ANYMOTION_API_URL)
+        api_url = cast(str, api_url)
+
+        if session is None:
+            session = HttpSession()
+        elif not isinstance(session, HttpSession):
             raise ClientValueError(
                 f"session is must be HttpSession class: {type(session)}"
             )
         self.session = session
 
-        parts = urlparse(api_url)
-        api_path = parts.path
-        if "anymotion" not in api_path:
-            raise ClientValueError(f"Invalid API URL: {api_url}")
-        if api_path[-1] != "/":
-            api_path += "/"
-
-        base_url = str(urlunparse((parts.scheme, parts.netloc, "", "", "", "")))
-        self._api_url = urljoin(base_url, api_path)
+        base_url, api_path = self._check_and_parse_api_url(api_url)
+        self._api_url: str = urljoin(base_url, api_path)
 
         self.auth = Authentication(
             client_id, client_secret, base_url=base_url, session=self.session
@@ -86,6 +92,7 @@ class Client(object):
         self._max_steps = int(max(1, timeout / self._interval))
 
         self._page_size = 1000
+        self._chunk_size = 1024 * 1024  # 1MB
 
     @check_endpoint
     def get_one_data(self, endpoint: str, endpoint_id: int) -> dict:
@@ -106,11 +113,23 @@ class Client(object):
         return response.json
 
     def get_image(self, image_id: int) -> dict:
-        """Get image data."""
+        """Get image data.
+
+        Gets a single image data.
+        If you want to get more than one image data, use `get_images`.
+
+        Same as requesting GET /images/{image_id} directly.
+        """
         return self.get_one_data("images", image_id)
 
     def get_movie(self, movie_id: int) -> dict:
-        """Get movie data."""
+        """Get movie data.
+
+        Gets a single movie data.
+        If you want to get more than one movie data, use `get_movies`.
+
+        Same as requesting GET /movies/{movie_id} directly.
+        """
         return self.get_one_data("movies", movie_id)
 
     def get_keypoint(self, keypoint_id: int, join_data: bool = False) -> dict:
@@ -160,7 +179,7 @@ class Client(object):
         return comparison
 
     @check_endpoint
-    def get_list_data(self, endpoint: str, params: dict = {}) -> List[dict]:
+    def get_list_data(self, endpoint: str, params: Optional[dict] = None) -> List[dict]:
         """Get list data.
 
         Args:
@@ -171,6 +190,9 @@ class Client(object):
         Raises:
             RequestsError: HTTP request fails.
         """
+        if params is None:
+            params = {}
+
         url = urljoin(self._api_url, f"{endpoint}/")
         params["size"] = self._page_size
         data: List[dict] = []
@@ -181,27 +203,27 @@ class Client(object):
             data += sub_data
         return data
 
-    def get_images(self, params: dict = {}) -> List[dict]:
+    def get_images(self, params: Optional[dict] = None) -> List[dict]:
         """Get image list."""
         return self.get_list_data("images", params=params)
 
-    def get_movies(self, params: dict = {}) -> List[dict]:
+    def get_movies(self, params: Optional[dict] = None) -> List[dict]:
         """Get movie list."""
         return self.get_list_data("movies", params=params)
 
-    def get_keypoints(self, params: dict = {}) -> List[dict]:
+    def get_keypoints(self, params: Optional[dict] = None) -> List[dict]:
         """Get keypoint list."""
         return self.get_list_data("keypoints", params=params)
 
-    def get_drawings(self, params: dict = {}) -> List[dict]:
+    def get_drawings(self, params: Optional[dict] = None) -> List[dict]:
         """Get drawing list."""
         return self.get_list_data("drawings", params=params)
 
-    def get_analyses(self, params: dict = {}) -> List[dict]:
+    def get_analyses(self, params: Optional[dict] = None) -> List[dict]:
         """Get analysis list."""
         return self.get_list_data("analyses", params=params)
 
-    def get_comparisons(self, params: dict = {}) -> List[dict]:
+    def get_comparisons(self, params: Optional[dict] = None) -> List[dict]:
         """Get comparisons list."""
         return self.get_list_data("comparisons", params=params)
 
@@ -258,15 +280,17 @@ class Client(object):
         path: Optional[Union[str, Path]] = None,
         exist_ok: bool = False,
         fix_suffix: bool = False,
+        stream: bool = True,
     ) -> Path:
         """Download a file from drawing_id.
 
         Args:
             drawing_id
-            path: output file path or directory path.
-            exist_ok: if false (default), FileExistsError is raised if the target file
+            path: The output file path or directory path.
+            exist_ok: If false (default), FileExistsError is raised if the target file
                 already exists.
             fix_suffix: If the extension of path is invalid, correct it.
+            stream: If true (default), steram download.
 
         Returns:
             The path to the downloaded file.
@@ -302,9 +326,13 @@ class Client(object):
             logger.error(f"File exists: {path}")
             raise FileExistsError(f"File exists: {path}")
 
-        response = self.session.request(url)
+        response = self.session.request(url, stream=stream)
         with path.open("wb") as f:
-            f.write(response.raw.content)
+            if stream:
+                for chunk in response.raw.iter_content(chunk_size=self._chunk_size):
+                    f.write(chunk)
+            else:
+                f.write(response.raw.content)
         logger.info(f"Download file to {path}.")
 
         return path
@@ -492,3 +520,14 @@ class Client(object):
         else:
             result.status = "TIMEOUT"
         return result
+
+    def _check_and_parse_api_url(self, api_url: str) -> Tuple[str, str]:
+        parts = urlparse(api_url)
+        api_path = parts.path
+        if "anymotion" not in api_path:
+            raise ClientValueError(f"Invalid API URL: {api_url}")
+        if api_path[-1] != "/":
+            api_path += "/"
+
+        base_url = str(urlunparse((parts.scheme, parts.netloc, "", "", "", "")))
+        return base_url, api_path
